@@ -4,13 +4,13 @@
 import logging
 import os
 import time
-import warnings
-from typing import Dict, List, Optional
+from typing import List, Optional
 
 import numpy as np
 from PIL import Image
 from tqdm import tqdm
 
+from sahi.auto_model import AutoDetectionModel
 from sahi.model import DetectionModel
 from sahi.postprocess.combine import (
     GreedyNMMPostprocess,
@@ -31,13 +31,9 @@ from sahi.utils.cv import (
     read_image_as_pil,
     visualize_object_predictions,
 )
-from sahi.utils.file import Path, import_class, increment_path, list_files, save_json, save_pickle
+from sahi.utils.file import Path, increment_path, list_files, save_json, save_pickle
+from sahi.utils.import_utils import check_requirements
 
-MODEL_TYPE_TO_MODEL_CLASS_NAME = {
-    "mmdet": "MmdetDetectionModel",
-    "yolov5": "Yolov5DetectionModel",
-    "detectron2": "Detectron2DetectionModel",
-}
 POSTPROCESS_NAME_TO_CLASS = {
     "GREEDYNMM": GreedyNMMPostprocess,
     "NMM": NMMPostprocess,
@@ -54,7 +50,6 @@ logger = logging.getLogger(__name__)
 def get_prediction(
     image,
     detection_model,
-    image_size: int = None,
     shift_amount: list = [0, 0],
     full_shape=None,
     postprocess: Optional[PostprocessPredictions] = None,
@@ -67,8 +62,6 @@ def get_prediction(
         image: str or np.ndarray
             Location of image or numpy image matrix to slice
         detection_model: model.DetectionMode
-        image_size: int
-            Inference input size.
         shift_amount: List
             To shift the box and mask predictions from sliced image to full
             sized image, should be in the form of [shift_x, shift_y]
@@ -84,16 +77,13 @@ def get_prediction(
             object_prediction_list: a list of ObjectPrediction
             durations_in_seconds: a dict containing elapsed times for profiling
     """
-    if image_size is not None:
-        warnings.warn("Set 'image_size' at DetectionModel init.", DeprecationWarning)
-
     durations_in_seconds = dict()
 
     # read image as pil
     image_as_pil = read_image_as_pil(image)
     # get prediction
     time_start = time.time()
-    detection_model.perform_inference(np.ascontiguousarray(image_as_pil), image_size=image_size)
+    detection_model.perform_inference(np.ascontiguousarray(image_as_pil))
     time_end = time.time() - time_start
     durations_in_seconds["prediction"] = time_end
 
@@ -128,7 +118,6 @@ def get_prediction(
 def get_sliced_prediction(
     image,
     detection_model=None,
-    image_size: int = None,
     slice_height: int = 512,
     slice_width: int = 512,
     overlap_height_ratio: float = 0.2,
@@ -148,8 +137,6 @@ def get_sliced_prediction(
         image: str or np.ndarray
             Location of image or numpy image matrix to slice
         detection_model: model.DetectionModel
-        image_size: int
-            Input image size for each inference (image is scaled by preserving asp. rat.).
         slice_height: int
             Height of each slice.  Defaults to ``512``.
         slice_width: int
@@ -190,8 +177,6 @@ def get_sliced_prediction(
             object_prediction_list: a list of sahi.prediction.ObjectPrediction
             durations_in_seconds: a dict containing elapsed times for profiling
     """
-    if image_size is not None:
-        warnings.warn("Set 'image_size' at DetectionModel init.", DeprecationWarning)
 
     # for profiling
     durations_in_seconds = dict()
@@ -244,7 +229,6 @@ def get_sliced_prediction(
         prediction_result = get_prediction(
             image=image_list[0],
             detection_model=detection_model,
-            image_size=image_size,
             shift_amount=shift_amount_list[0],
             full_shape=[
                 slice_image_result.original_image_height,
@@ -265,7 +249,6 @@ def get_sliced_prediction(
         prediction_result = get_prediction(
             image=image,
             detection_model=detection_model,
-            image_size=image_size,
             shift_amount=[0, 0],
             full_shape=None,
             postprocess=None,
@@ -466,9 +449,8 @@ def predict(
     # init model instance
     time_start = time.time()
     if detection_model is None:
-        model_class_name = MODEL_TYPE_TO_MODEL_CLASS_NAME[model_type]
-        DetectionModel = import_class(model_class_name)
-        detection_model = DetectionModel(
+        detection_model = AutoDetectionModel.from_local(
+            model_type=model_type,
             model_path=model_path,
             config_path=model_config_path,
             confidence_threshold=model_confidence_threshold,
@@ -492,7 +474,8 @@ def predict(
     ):
         # get filename
         if source_is_video:
-            relative_filepath = ".png"
+            video_name = Path(source).stem
+            relative_filepath = video_name + "_frame_" + str(ind)
         elif os.path.isdir(source):  # preserve source folder structure in export
             relative_filepath = str(Path(image_path)).split(str(Path(source)))[-1]
             relative_filepath = relative_filepath[1:] if relative_filepath[0] == os.sep else relative_filepath
@@ -540,7 +523,7 @@ def predict(
         tqdm.write("Prediction time is: {:.2f} ms".format(prediction_result.durations_in_seconds["prediction"] * 1000))
 
         if dataset_json_path:
-            if source_is_video == True:
+            if source_is_video is True:
                 raise NotImplementedError("Video input type not supported with coco formatted dataset json")
 
             # append predictions in coco format
@@ -665,6 +648,7 @@ def predict(
         return {"export_dir": save_dir}
 
 
+@check_requirements(["fiftyone"])
 def predict_fiftyone(
     model_type: str = "mmdet",
     model_path: str = None,
@@ -749,10 +733,8 @@ def predict_fiftyone(
     from sahi.utils.fiftyone import create_fiftyone_dataset_from_coco_file, fo
 
     # assert prediction type
-    assert (
-        no_standard_prediction and no_sliced_prediction
-    ) is not True, "'no_standard_pred' and 'no_sliced_prediction' cannot be True at the same time."
-
+    if no_standard_prediction and no_sliced_prediction:
+        raise ValueError("'no_standard_pred' and 'no_sliced_prediction' cannot be True at the same time.")
     # for profiling
     durations_in_seconds = dict()
 
@@ -760,9 +742,8 @@ def predict_fiftyone(
 
     # init model instance
     time_start = time.time()
-    model_class_name = MODEL_TYPE_TO_MODEL_CLASS_NAME[model_type]
-    DetectionModel = import_class(model_class_name)
-    detection_model = DetectionModel(
+    detection_model = AutoDetectionModel.from_local(
+        model_type=model_type,
         model_path=model_path,
         config_path=model_config_path,
         confidence_threshold=model_confidence_threshold,
